# type: ignore
from flask import request, current_app

from covador.flask import list_schema
from covador.utils import parse_qs
from covador.vdecorator import ValidationDecorator, ErrorHandler
from covador.errors import error_to_json


def get_form(*_args, **_kwargs):
    try:
        return request._covador_form
    except AttributeError:
        ctype = request.content_type or ''
        if ctype.startswith('multipart/form-data'):
            form = request.form.to_dict(False)
        elif ctype.startswith('application/x-www-form-urlencoded'):
            # There appears to be a bug in the Covador code: https://github.com/baverman/covador/blob/master/covador/flask.py#L32
            # It's unclear why Covador checks only request.data when the content type is 'application/x-www-form-urlencoded'.
            # After enabling Flask-WTF and passing the CSRF token, Flask seems to pull the necessary data from `request.data`,
            # emptying it and moving the data into `request.form`. We can remove this file once the bug is fixed in the github.
            if data := request.get_data(parse_form_data=False):
                form = parse_qs(data, process_semicolons=False)
            else:
                form = request.form.to_dict(False)
        else:
            form = {}
        request._covador_form = form
        return form


@ErrorHandler
def error_handler(ctx):
    return current_app.response_class(error_to_json(ctx.exception), mimetype='application/json', status=400)


form = ValidationDecorator(get_form, error_handler, list_schema)
