from typing import TypeVar, List, Generic, Tuple, Optional, Any, Dict

from wtforms.form import Form
from flask_admin.base import BaseView
from flask_admin.model.base import ViewArgs
from jinja2.runtime import Context
from sqlalchemy.orm import Query, Session, scoped_session
from .filters import BaseSQLAFilter

Model = TypeVar('Model')

class ModelView(BaseView, Generic[Model]):
    def __init__(self, view: type[Model], session: scoped_session[Session], name: str): ...
    def _export_data(self) -> Tuple[int, List[Model]]: ...
    def get_list_value(self, context: Context, model: Model, name: str) -> Any: ...
    def get_detail_value(self, context: Context, model: Model, name: str) -> Any: ...
    def get_export_value(self, model: Model, name: str) -> Any: ...
    def get_one(self, id: str) -> Optional[Model]: ...
    def get_query(self) -> Query: ...
    def get_count_query(self) -> Query: ...
    def handle_view_exception(self, exc: Exception) -> bool: ...
    def edit_form(self, obj: Optional[Model] = None) -> Form: ...
    def create_form(self) -> Form: ...
    def on_model_change(self, form: Form, model: Model, is_created: bool) -> None: ...
    def delete_model(self, model: Model) -> bool: ...
    def handle_filter(self, filter: BaseSQLAFilter) -> BaseSQLAFilter: ...
    def init_search(self) -> bool: ...
    def _get_list_extra_args(self) -> ViewArgs: ...
    def _apply_search(
        self,
        query: Query,
        count_query: Optional[Query],
        joins: Dict[Any, Any],
        count_joins: Optional[Dict[Any, Any]],
        search: str,
    ) -> Tuple[Query, Optional[Query], Dict[Any, Any], Optional[Dict[Any, Any]]]: ...
    def _apply_filters(
        self,
        query: Query,
        count_query: Optional[Query],
        joins: Dict[Any, Any],
        count_joins: Optional[Dict[Any, Any]],
        filters: Optional[List[Any]],
    ) -> Tuple[Query, Optional[Query], Dict[Any, Any], Optional[Dict[Any, Any]]]: ...

    _filter_joins: Dict[Any, List[Any]]
    _filters: List[Any]
    _auto_joins: List[Any]
    _search_fields: List[Tuple[Any, List[Any]]]
    _search_supported: bool
